//+------------------------------------------------------------------+
//|                                                    Simulator.mq5 |
//|                        Copyright 2019, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
//---
#define Magic_Number 0
//---
#include <Trade\PositionInfo.mqh>
#include <Trade\Trade.mqh>
//---
CPositionInfo  m_position; // trade position object
CTrade         m_trade;    // trading object
//---
input double   InpLots     = 0.01; // Lots
input int      Stop_Loss   = 0;    // Stop_Loss
input int      Take_Profit = 0;    // Take_Profit
input int      Trail_Start = 0;    // Take_Profit
input int      Trail_Size  = 0;    // Take_Profit
//---
bool   Manual_Confirm    =false;
int    Retry_Delay       =1000;
int    Retry_Times       =20;
string Order_Comment     ="XXX";
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   m_trade.SetTypeFillingBySymbol(_Symbol);
   m_trade.SetExpertMagicNumber(Magic_Number);
   m_trade.SetDeviationInPoints(0);
   m_trade.SetMarginMode();
//---
   bool res=false;
     {
      ObjectCreate(0,"BUY",OBJ_BUTTON,0,0,0);
      ObjectSetInteger(0,"BUY",OBJPROP_XDISTANCE,ChartGetInteger(0,CHART_WIDTH_IN_PIXELS)-102);
      ObjectSetInteger(0,"BUY",OBJPROP_YDISTANCE,37);
      ObjectSetString(0,"BUY",OBJPROP_TEXT,"BUY");
      ObjectSetInteger(0,"BUY",OBJPROP_BGCOLOR,clrMediumSeaGreen);

      ObjectCreate(0,"SELL",OBJ_BUTTON,0,0,0);
      ObjectSetInteger(0,"SELL",OBJPROP_XDISTANCE,ChartGetInteger(0,CHART_WIDTH_IN_PIXELS)-50);
      ObjectSetInteger(0,"SELL",OBJPROP_YDISTANCE,37);
      ObjectSetString(0,"SELL",OBJPROP_TEXT,"SELL");
      ObjectSetInteger(0,"SELL",OBJPROP_BGCOLOR,clrDarkOrange);

      ObjectCreate(0,"CLOSE",OBJ_BUTTON,0,0,0);
      ObjectSetInteger(0,"CLOSE",OBJPROP_XDISTANCE,ChartGetInteger(0,CHART_WIDTH_IN_PIXELS)-75);
      ObjectSetInteger(0,"CLOSE",OBJPROP_YDISTANCE,17);
      ObjectSetString(0,"CLOSE",OBJPROP_TEXT,"CLOSE");
      ObjectSetInteger(0,"CLOSE",OBJPROP_BGCOLOR,clrMagenta);
     }
   res=true;
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   ObjectsDeleteAll(0,"SELL");
   ObjectsDeleteAll(0,"BUY");
   ObjectsDeleteAll(0,"CLOSE");
//---
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   if(ObjectGetInteger(0,"BUY",OBJPROP_STATE)!=0)
     {
      ObjectSetInteger(0,"BUY",OBJPROP_STATE,0);
      //--- check for long position (BUY) possibility
      TradeBuy(InpLots,false);
     }
   if(ObjectGetInteger(0,"SELL",OBJPROP_STATE)!=0)
     {
      ObjectSetInteger(0,"SELL",OBJPROP_STATE,0);
      //--- check for short position (SELL) possibility
      TradeSell(InpLots,false);
     }
   if(ObjectGetInteger(0,"CLOSE",OBJPROP_STATE)!=0)
     {
      ObjectSetInteger(0,"CLOSE",OBJPROP_STATE,0);
      //--- check for short position (SELL) possibility
      TradeCloseAll(false);
     }
   RunTrailing();
//---
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void TradeBuy(double lot,bool auto)
  {
   bool check=false;
   if(!auto && Manual_Confirm)
     {
      string text="OPENING LONG: "+_Symbol+" "+string(lot)+" LOTS";
      if(MessageBox(text,"",MB_OKCANCEL|MB_ICONINFORMATION)==IDCANCEL)
         return;
     }
   for(int i=0; i<Retry_Times; i++)
     {
      double price=SymbolInfoDouble(_Symbol,SYMBOL_ASK);
      double SL=(Stop_Loss==0)   ? 0 : NormalizeDouble(price-Stop_Loss*_Point,_Digits);
      double TP=(Take_Profit==0) ? 0 : NormalizeDouble(price+Take_Profit*_Point,_Digits);
      check=m_trade.Buy(lot,_Symbol,price,SL,TP,Order_Comment);
      if(check)
         break;
      Print("#",Magic_Number,": TRADING ERROR ",m_trade.ResultRetcode());
      Sleep(Retry_Delay);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void TradeSell(double lot,bool auto)
  {
   bool check=false;
   if(!auto && Manual_Confirm)
     {
      string text="OPENING SHORT: "+_Symbol+" "+string(lot)+" LOTS";
      if(MessageBox(text,"",MB_OKCANCEL|MB_ICONINFORMATION)==IDCANCEL)
         return;
     }
   for(int i=0; i<Retry_Times; i++)
     {
      double price=SymbolInfoDouble(_Symbol,SYMBOL_BID);
      double SL=(Stop_Loss==0)   ? 0 : NormalizeDouble(price+Stop_Loss*_Point,_Digits);
      double TP=(Take_Profit==0) ? 0 : NormalizeDouble(price-Take_Profit*_Point,_Digits);
      check=m_trade.Sell(lot,_Symbol,price,SL,TP,Order_Comment);
      if(check)
         break;
      Print("#",Magic_Number,": TRADING ERROR ",m_trade.ResultRetcode());
      Sleep(Retry_Delay);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void TradeCloseAll(bool auto)
  {
   if(!auto && Manual_Confirm)
     {
      string text="CLOSING ALL POSITIONS: "+_Symbol;
      if(MessageBox(text,"",MB_OKCANCEL|MB_ICONINFORMATION)==IDCANCEL)
         return;
     }
   for(int i=PositionsTotal()-1; i>=0; i--)
     {
      if(!m_position.SelectByIndex(i))
         continue;
      if(m_position.Magic()!=Magic_Number)
         continue;
      if(m_position.Symbol()!=_Symbol)
         continue;
      TradeClose(m_position.Ticket());
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void TradeClose(long ticket)
  {
   bool check=false;
   for(int i=0; i<Retry_Times; i++)
     {
      check=m_trade.PositionClose(ticket);
      if(check)
         break;
      Print("#",Magic_Number,": TRADING ERROR ",m_trade.ResultRetcode());
      Sleep(Retry_Delay);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void RunTrailing()
  {
   if(Trail_Start==0)
      return;
   if(Trail_Size==0)
      return;
   for(int i=PositionsTotal()-1; i>=0; i--)
     {
      if(!m_position.SelectByIndex(i))
         continue;
      if(m_position.Magic()!=Magic_Number)
         continue;
      if(m_position.Symbol()!=_Symbol)
         continue;

      double open=m_position.PriceOpen();
      double take=m_position.TakeProfit();
      double stop=m_position.StopLoss();
      int type=m_position.PositionType();

      if(type==POSITION_TYPE_BUY)
        {
         double price=SymbolInfoDouble(_Symbol,SYMBOL_BID);
         double trail=NormalizeDouble(price-Trail_Size*_Point,_Digits);
         double trigger=NormalizeDouble(open+Trail_Start*_Point,_Digits);
         if(price>=trigger)
            if(trail>stop || stop==0)
               TradeModify(m_position.Ticket(),open,trail,take);
        }
      if(type==POSITION_TYPE_SELL)
        {
         double price=SymbolInfoDouble(_Symbol,SYMBOL_ASK);
         double trail=NormalizeDouble(price+Trail_Size*_Point,_Digits);
         double trigger=NormalizeDouble(open-Trail_Start*_Point,_Digits);
         if(price<=trigger)
            if(trail<stop || stop==0)
               TradeModify(m_position.Ticket(),open,trail,take);
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void TradeModify(long ticket,double open,double stop,double take)
  {
   bool check=false;
   for(int i=0; i<Retry_Times; i++)
     {
      check=m_trade.PositionModify(ticket,stop,take);
      if(check)
         break;
      Print("#",Magic_Number,": TRADING ERROR ",m_trade.ResultRetcode());
      Sleep(Retry_Delay);
     }
  }
//+------------------------------------------------------------------+
